/***
 * Contains basic SlickGrid editors.
 * @module Editors
 * @namespace Slick
 */

define(['src/util/util', 'components/spectrum/spectrum', 'jquery'], function(Util) {
    Util.loadCss("./components/spectrum/spectrum.css");
    (function ($) {
        // register namespace
        $.extend(true, window, {
            "Slick": {
                "Editors": {
                    "TextValue": TextValueEditor,
                    "ColorValue": ColorEditor,
                    "Text": TextValueEditor,
                    "SpecialNativeObject": SpecialNativeObjectEditor,
                    "DataNumberEditor": DataNumberEditor,
                    "DataBooleanEditor": DataBooleanEditor
                }
            }
        });

        function ColorEditor(args) {
            var $cont, $input, defaultValue;
            this.init = function() {
                $cont = $('<div/>');
                $cont.append('<input type="text">');
                $input = $cont.find('input');
                $input.appendTo(args.container)
                    .bind("keydown.nav", function (e) {
                        if (e.keyCode === $.ui.keyCode.LEFT || e.keyCode === $.ui.keyCode.RIGHT) {
                            e.stopImmediatePropagation();
                        }
                    })
                    .focus()
                    .select();
                $input.spectrum({
                    color: $input.val(),
                    preferredFormat: 'hex',
                    change: function(color) {
                        console.log('change');
                        console.log('color: ', color);
                        $input.spectrum('hide');
                        args.commitChanges();
                    },
                    move: function(color) {
                        console.log('move');
                        console.log('color: ', color);
                    },
                    show: function() {
                        console.log('show');
                    },
                    hide: function() {
                        console.log('hide');
                        args.commitChanges();
                    },
                    localStorageKey: 'visualizer-spectrum',
                    showPalette: true,
                    showSelectionPalette: true,
                    palette: []
                });

                $input.next().first().click();
            };

            this.destroy = function() {
                $cont.remove();
            };

            this.focus = function() {
                $input.focus();
            };

            this.getValue = function() {
                $input.val();
            };

            this.setValue = function(val) {
                $input.val(val);
            };

            this.loadValue = function(item) {
                defaultValue = item.getChildSync(args.column.jpath);
                if(defaultValue) {
                    defaultValue = defaultValue.value || '#000000';
                }
                else {
                    defaultValue = '#000000';
                }
                $input.val(defaultValue);
                $input.spectrum('set', defaultValue);
                $input[0].defaultValue = defaultValue;
                $input.select();
            };

            this.serializeValue = function () {
                return $input.val();
            };


            this.applyValue = function (item, state) {
                var isNew = _.isEmpty(item);
                DataObject.check(item, true);
                var newState = {
                    type: 'color',
                    value: state
                };


                if(isNew) {
                    var newItem = {};
                    newItem[args.grid.module.view.idPropertyName] = 'id_'+args.grid.module.view.getNextIncrementalId();
                    newItem = DataObject.check(newItem, true);
                    newItem.setChildSync(args.column.jpath, newState);
                    args.grid.module.view.slick.data.addItem(newItem);
                    return newState;
                }
                else {
                    args.grid.module.model.dataSetChildSync(item, args.column.jpath, newState);
                }


            };

            this.isValueChanged = function () {
                return (!($input.val() == "" && defaultValue == null)) && ($input.val() != defaultValue);
            };

            this.validate = function () {
                if (args.column.validator) {
                    var validationResults = args.column.validator($input.val());
                    if (!validationResults.valid) {
                        return validationResults;
                    }
                }

                return {
                    valid: true,
                    msg: null
                };
            };

            this.init();
        }

        function TextValueEditor(args) {
            this.args = args;
            this.init = defaultInit;
            this.destroy = defaultDestroy;
            this.focus = defaultFocus;
            this.getValue = defaultGetValue;
            this.setValue = defaultSetValue;
            this.loadValue = defaultLoadValue;
            this.serializeValue = defaultSerializeValue;
            this.isValueChanged = defaultIsValueChanged;
            this.validate = defaultValidate;

            this.applyValue = function(item, state) {
                var isNew = _.isEmpty(item);
                DataObject.check(item, true);
                var newState = {
                    type: this.args.column.dataType,
                    value: state
                };


                if(isNew) {
                    var newItem = {};
                    newItem[this.args.grid.module.view.idPropertyName] = 'id_'+this.args.grid.module.view.getNextIncrementalId();
                    newItem = DataObject.check(newItem, true);
                    newItem.setChildSync(this.args.column.jpath, newState);
                    this.args.grid.module.view.slick.data.addItem(newItem);
                    return newState;
                }
                else {
                    this.args.grid.module.model.dataSetChildSync(item, this.args.column.jpath, newState);
                }
            };

            this.init();
        }


        function SpecialNativeObjectEditor(args) {
            this.args = args;
            this.init = defaultInit;
            this.destroy = defaultDestroy;
            this.focus = defaultFocus;
            this.getValue = defaultGetValue;
            this.setValue = defaultSetValue;
            this.loadValue = defaultLoadValue;
            this.serializeValue = defaultSerializeValue;
            this.applyValue = defaultApplyValue;
            this.isValueChanged = defaultIsValueChanged;
            this.validate = defaultValidate;
            this.init();
        }

        function DataNumberEditor(args) {
            this.args = args;
            this.init = defaultInit;
            this.destroy = defaultDestroy;
            this.focus = defaultFocus;
            this.getValue = numberGetValue;
            this.setValue = defaultSetValue;
            this.loadValue = defaultLoadValue;
            this.serializeValue = defaultSerializeValue;
            this.applyValue = numberApplyValue;
            this.isValueChanged = defaultIsValueChanged;
            this.validate = defaultValidate;
            this.init();
        }

        function DataBooleanEditor(args) {
            this.args = args;
            this.init = booleanInit;
            this.destroy = defaultDestroy;
            this.focus = defaultFocus;
            this.getValue = numberGetValue;
            this.setValue = defaultSetValue;
            this.loadValue = booleanLoadValue;
            this.serializeValue = booleanSerializeValue;
            this.applyValue = booleanApplyValue;
            this.isValueChanged = booleanIsValueChanged;
            this.validate = defaultValidate;
            this.init();
        }
    })(jQuery);


    // ======== DEFAULT EDITOR FUNCTIONS ===============
    function defaultValidate() {
        if (this.args.column.validator) {
            var validationResults = this.args.column.validator(this.serializeValue());
            if (!validationResults.valid) {
                return validationResults;
            }
        }

        return {
            valid: true,
            msg: null
        };
    }

    function defaultIsValueChanged() {
        return (!(this.$input.val() == "" && this.defaultValue == null)) && (this.$input.val() != this.defaultValue);
    }

    function defaultApplyValue(item, state) {
        var isNew = _.isEmpty(item);
        DataObject.check(item, true);
        var newState = state;


        if(isNew) {
            var newItem = {};
            newItem[this.args.grid.module.view.idPropertyName] = 'id_'+this.args.grid.module.view.getNextIncrementalId();
            newItem = DataObject.check(newItem, true);
            newItem.setChildSync(this.args.column.jpath, state);
            this.args.grid.module.view.slick.data.addItem(newItem);
            return newState;
        }
        else {
            this.args.grid.module.model.dataSetChildSync(item, this.args.column.jpath, newState);
        }
    }

    function defaultSerializeValue() {
        return this.$input.val();
    }

    function defaultLoadValue(item) {
        this.defaultValue = item.getChildSync(this.args.column.jpath);
        this.defaultValue = this.defaultValue ? this.defaultValue.get() || "" : "";
        this.$input.val(this.defaultValue);
        this.$input[0].defaultValue = this.defaultValue;
        this.$input.select();
    }

    function defaultSetValue(val) {
        this.$input.val(val);
    }

    function defaultGetValue() {
        return this.$input.val();
    }

    function defaultInit() {
        this.$input = $("<INPUT type=text class='editor-text' />")
            .appendTo(this.args.container)
            .bind("keydown.nav", function (e) {
                if (e.keyCode === $.ui.keyCode.LEFT || e.keyCode === $.ui.keyCode.RIGHT) {
                    e.stopImmediatePropagation();
                }
            })
            .focus()
            .select();
    }

    function defaultDestroy() {
        this.$input.remove();
    }

    function defaultFocus() {
        this.$input.focus();
    }

    // =========== DATA NUMBER ===============
    function numberGetValue() {
        return +this.$input.val();
    }

    function numberApplyValue(item, state) {
        state = +state;
        return defaultApplyValue.call(this, item, state);
    }

    // =========== DATA BOOLEAN ==============
    function booleanInit() {
        this.$input = $("<INPUT type=checkbox value='true' class='editor-checkbox' hideFocus>");
        this.$input.appendTo(this.args.container);
        this.$input.focus();
    }

    function booleanLoadValue(item) {
        this.defaultValue = item.getChildSync(this.args.column.jpath);
        if (this.defaultValue) {
            this.$input.attr("checked", "checked");
        } else {
            this.$input.removeAttr("checked");
        }
    }

    function booleanSerializeValue() {
        return !!this.$input.attr("checked");
    }

    function booleanIsValueChanged() {
        return this.serializeValue !== this.defaultValue;
    }

    function booleanApplyValue(item, state) {
        state = !!state;
        defaultApplyValue.call(this, item, state);
    }


});

